#!/usr/bin/env python
# -*- coding: utf-8 -*-

from .base import BaseDisplayPlugin
from .style_classic import DisplayClassic
from .style_modern import DisplayModern
from .style_minimal import DisplayMinimal
from .style_agent import DisplayAgent
from .manager import DisplayManager
from .live_display import LiveDisplay

__all__ = [
    'BaseDisplayPlugin',
    'DisplayClassic',
    'DisplayModern',
    'DisplayMinimal',
    'DisplayAgent',
    'DisplayManager',
    'LiveDisplay'
] 